#!/usr/bin/env python
# -*- coding:utf-8 _*-  
"""
@author  : Lin Luo / Bruce Liu
@time    : 2020/1/20 14:09
@contact : 15869300264@163.com
"""
from .cad_util import CadUtil, CadException


class AppConfigUtil(object):
    def __init__(self, app_id: int):
        self._cad_client = CadUtil()
        self._app_id = app_id

    def get_sk(self, ak, dict_obj: dict = None) -> str or None:
        """
        根据ak获取sk
        :param ak:
        :param dict_obj: redis中的用户信息，如果没有传入，则从redis中读取
        :return: 如果后去失败，则返回None
        """
        if dict_obj is None:
            try:
                dict_obj = self._cad_client.get(ak)
            except CadException:
                dict_obj = None

            if dict_obj is None:
                return None
        else:
            return dict_obj.get('sk')

    def check(self, ak: str, sk: str or None, auth_code: str = None, dict_obj: dict = None) -> bool:
        """
        校验ak权限
        :param ak:
        :param sk: 如果sk传入None，则跳过sk判断过程
        :param auth_code: 权限code ，如果没有传入，则只判断是否存在应用名称
        :param dict_obj: redis中的用户信息，如果没有传入，则从redis中读取
        :return:
        """
        # 判断是否传入用户信息
        if dict_obj is None:
            # 如果没有传入，则从redis中读取
            dict_obj = self._cad_client.get(ak)
            # 判断读取的数据是否为None
            if dict_obj is None:
                # 如果为None，则返回False
                return False
        # 如果sk不为None，判断aksk是否有效
        if sk is not None and dict_obj.get('sk') != sk:
            return False
        permissions = dict_obj.get('permissions', {})
        # 判断auth_code是否为None
        if auth_code is None:
            # 如果为None，则判断是否存在对应应用
            return self._app_id in permissions['app_ids']
        else:
            # 如果不为None，则判断应用及权限
            if self._app_id in permissions['app_ids'] and auth_code in permissions['auth_codes']:
                return True
            else:
                return False



#!/usr/bin/env python
# -*- coding:utf-8 _*-  
"""
@license : Copyright(C), WAYZ
@author  : Bruce Liu
@time    : 2020/1/22 16:25
@contact : bruce.liu@wayz.ai
"""
from unittest import TestCase

from cad.app_config_util import AppConfigUtil

ak = '958c6d6b8ee85d9e'
sk = 'e201737ee515c7b8'


class TestAppConfigUtil(TestCase):
    def test_get_dict_success(self):
        """

        :return:
        """
        instance = AppConfigUtil(2)
        res = instance.check(ak, sk)
        self.assertEqual(res, True)
        res = instance.check(ak, sk, auth_code='234')
        self.assertEqual(res, True)
        res = instance.check(ak, '123')
        self.assertEqual(res, False)
        res = instance.check(ak, None)
        self.assertEqual(res, True)



#!/usr/bin/env python
# -*- coding:utf-8 _*-  
"""
@author  : Lin Luo / Bruce Liu
@time    : 2020/2/4 10:28
@contact : 15869300264@163.com
"""
from json import loads

from dynaconf import settings as dynasettings

from .aes_util import AESUtil
from .redis_util import RedisUtil


class CadException(BaseException):
    def __init__(self, ak):
        self._msg = f'the ak: {ak} is not found'

    def __str__(self):
        return self._msg


class CadUtil(object):
    def __init__(self, redis_url_str: str = None, key: str = None, cache_key: str = None):
        """

        :param redis_url_str:
        :param key:
        :param cache_key:
        """
        if redis_url_str is None:
            redis_url_str = dynasettings.REDIS_ADDRESS
        if key is None:
            key = dynasettings.COMMON_SALT
        if cache_key is None:
            self._cache_key = f'{dynasettings.APPLICATION_NAME}:{dynasettings.APP_CONFIG_PREFIX}'
        else:
            self._cache_key = cache_key
        self._redis_client = RedisUtil(url_str=redis_url_str).redis
        self._aes_client = AESUtil(encrypt_key=key)

    def get(self, ak: str) -> dict:
        """
        通过ak获取解密后的用户信息
        :param ak:
        :return:
        """
        data = self._redis_client.get(f'{self._cache_key}:{ak}')
        if data is not None:
            data = str(data, 'utf-8')
            return loads(self._aes_client.decrypt(data))
        else:
            raise CadException(ak)



#!/usr/bin/env python
# -*- coding:utf-8 _*-  
"""
@author  : Lin Luo / Bruce Liu
@time    : 2020/1/3 11:01
@contact : 15869300264@163.com
"""

from redis import ConnectionPool, Redis


class RedisUtil(object):
    def __init__(self, host: str = 'localhost', port: int = 6379, db: int = 0, password: str = '', url_str: str = None,
                 **kwargs):
        if url_str is not None:
            hosts = url_str.split(':')
            host = hosts[0]
            ports = hosts[1].split('/')
            port = ports[0]
            db = ports[1]
        self._pool = ConnectionPool(host=host, port=port, db=db, password=password, **kwargs)
        self._redis = Redis(connection_pool=self._pool)

    @property
    def redis(self):
        return self._redis

    @property
    def pool(self):
        return self._pool



#!/usr/bin/env python
# -*- coding:utf-8 _*-  
"""
@license : Copyright(C), WAYZ
@author  : Bruce Liu
@time    : 2020/2/4 10:39
@contact : bruce.liu@wayz.ai
"""
from cad.cad_util import CadUtil
from unittest import TestCase

ak_list = [
    '958c6d6b8ee85d9e',
    '51b8125330a72b59',
]


class TestCadUtil(TestCase):
    def test_get_ak(self):
        """

        :return:
        """
        client = CadUtil()
        for ak in ak_list:
            data = client.get(ak)
            self.assertIsInstance(data, dict)
            print(data)



#!/usr/bin/env python
# -*- coding:utf-8 _*-  
"""
@author  : Lin Luo / Bruce Liu
@time    : 2020/1/20 14:09
@contact : 15869300264@163.com
"""
import base64

from Crypto.Cipher import AES


class AESUtil(object):
    def __init__(self, encrypt_key: str, mode=AES.MODE_ECB, iv=''):
        """

        :param encrypt_key:
        :param mode:
        """
        self._key = self._data_format(encrypt_key)
        self._mode = mode
        self._iv = iv
        self._aes_client = self._get_new_cipher()

    def _get_new_cipher(self):
        """

        :return:
        """
        return AES.new(self._key, self._mode)

    @staticmethod
    def _data_format(data):
        """

        :param data:
        :return:
        """
        data = str(data)
        lack = len(data) % 16
        if lack != 0:
            data += ('\0' * (16 - lack))
        return data.encode('utf-8')

    def encrypt(self, data) -> bytes:
        """

        :param data:
        :return:
        """
        data = self._data_format(data)
        res = self._aes_client.encrypt(data)
        return base64.encodebytes(res).decode('utf-8')

    def decrypt(self, data) -> str:
        """

        :param data:
        :return:
        """
        data = base64.decodebytes(data.encode('utf-8'))
        res = self._aes_client.decrypt(data)
        tmp = res.replace(b'\x00', b'')
        return tmp.decode('utf-8')



# encoding: utf-8
"""
##基于redis的AES加解密工具包

2/4/2020   Bruce  0.8.2   初始版本<br/>
2/5/2020   Bruce  0.8.3   补充调用及配置说明，更新引用方式<br/>
2/5/2020   Bruce  0.9.1   新增鉴权模块<br/>
3/9/2020   Bruce  0.9.9   bugfix<br/>

###使用方法：<br/>
####1、配置全局变量<br/>
变量名|类型|用途|示例
REDIS_ADDRESS|字符串|redis服务器地址,需要带上ip、端口、库号|127.0.0.1:6379/1
COMMON_SALT|字符串|解密密钥|0648ea5562efffee
APPLICATION_NAME|字符串|cad应用名|mms
APP_CONFIG_PREFIX|字符串|cad配置前缀|app-config

####2、引入模块<br/>
from cad.cad_util import CadUtil  # 数据获取模块
from cad.app_config_util import AppConfigUtil  # 用户鉴权模块

####3、使用<br/>
`cad_util = CadUtil()`<br/>
`res = util.get(ak)`<br/>

ak为应用对应的ak键<br/>
res 即得到的解析结果<br/>
结果格式如下<br/>
`{
    "ak": "0648ea5562efffee",
    "sk": "1e058d312eb61448",
    "permissions": {
        "app_1": ["auth_1", "auth_2"],
        "app_2': ["auth_3", "auth_4"]
    }
}`
格式说明:<br/>
- ak: 字符串,应用的ak
- sk: 字符串,应用的sk
- permissions: 字典,应用的权限清单
  - permissions的每个键为该ak所拥有的其他应用的权限的应用名称
  - permissions每个键的值为该ak所拥有的对应应用的权限列表


`app_config_util = AppConfigUtil()pp_id`<br/>
校验ak是否有某应用的某个权限权限
app_config_util.check(ak, sk, auth_code)
校验ak是否有某应用的任意权限
app_config_util.check(ak,sk)

参数说明：<br/>
- ak: 传入的ak
- sk: 传入的sk
- app_id: 判断ak是否有该应用的权限
- auth_code: 判断ak是否有app下auth_code的权限，如果不传则判断ak是否有app的任意权限

结果说明：<br/>
True: 校验成功
False: 校验失败，原因如下：
- aksk不匹配
- ak没有对应app的权限
- ak没有对应app的auth_code权限
抛出异常CadException：<br/>
- ak不存在
"""
import sys
from setuptools import setup, find_packages
import cad

SHORT = 'cache aes decrypter'

if sys.platform == 'win32':
    setup(
        name='cad',
        version=cad.__version__,
        packages=find_packages(),
        install_requires=[
            'requests', 'redis', 'dynaconf', 'pycryptodome'
        ],
        long_description_content_type="text/markdown",
        url='',
        author=cad.__author__,
        author_email=cad.__email__,
        classifiers=[
            'Programming Language :: Python :: 3',
        ],
        include_package_data=True,
        package_data={'': ['*.py', '*.pyc']},
        zip_safe=False,
        platforms='any',

        description=SHORT,
        long_description=__doc__,
    )
else:
    setup(
        name='cad',
        version=cad.__version__,
        packages=find_packages(),
        install_requires=[
            'requests', 'redis', 'dynaconf', 'pycrypto'
        ],
        long_description_content_type="text/markdown",
        url='',
        author=cad.__author__,
        author_email=cad.__email__,
        classifiers=[
            'Programming Language :: Python :: 3',
        ],
        include_package_data=True,
        package_data={'': ['*.py', '*.pyc']},
        zip_safe=False,
        platforms='any',

        description=SHORT,
        long_description=__doc__,
    )



#!/usr/bin/env python
# -*- coding:utf-8 _*-  
"""
@license : Copyright(C), WAYZ
@author  : Bruce Liu
@time    : 2020/2/5 17:00
@contact : bruce.liu@wayz.ai
"""
from unittest import TestCase

from cad.aes_util import AESUtil


class TestAESUtil(TestCase):
    aes_util = AESUtil('123')

    def test_1(self):
        data = {
            'al': 123,
            'sk': "12dasfw4r"
        }

        en = self.aes_util.encrypt(data)
        print(en)
        de = self.aes_util.decrypt(en)
        print(de)
